//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                Contains Bloom Depth                 //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Prod80: Author of Adaptation Curve                  //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx or enbbloom.fx shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Empty_Row1 <string UIName=" ";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};
float Row_BloomDepth <
  string UIName="---------------------------BLOOM";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;
> = {0.0};

//{
  bool ENABLE_BLOOMDEPTH <
    string UIName = "Enable Bloom Depth Control";
  > = {false};
    float4 fBloomFNearDepth_E <
      string UIName="BLOOM: From Near Depth [E]-";  string UIWidget="Spinner";  float UIMin=1.0;  float UIMax=100000.0;  float UIStep=1.0;
    > = {200.0, 200.0, 200.0, 200.0};
//}

// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msBloomDepth(float3 inBloom, float2 inCoord)
{
/// Time and Location interpolators ++++++++++++++++++++++++++++++++++++++++++++++++++++++///
  float fBloomFNearDepth = TODE1(TODfactors(), fBloomFNearDepth_E.x, fBloomFNearDepth_E.y, fBloomFNearDepth_E.z, fBloomFNearDepth_E.w);
///+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++///

    float Depth    = TextureDepth.Sample(Sampler0, inCoord.xy ).x;
    float linDepth = linearDepth(Depth, 0.5, fBloomFNearDepth);
      inBloom.xyz  = lerp(inBloom.xyz, inBloom.xyz * linDepth, ENABLE_BLOOMDEPTH);

  return inBloom;
}