//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                 Contains HDR Toning                 //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Erik Reinhard: Photographic Tone Reproduction       //
// Michael Stark: Photographic Tone Reproduction       //
// Peter Shirley: Photographic Tone Reproduction       //
// James Ferwerda: Photographic Tone Reproduction      //
// Jim Hejl: Filmic ALU                                //
// Richard Burgess-Dawson: Filmic ALU                  //
// Charles Poynton: Color FAQ                          //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//

// This helper file is specifically only for use in the enbeffect.fx shader file!
// The below list is only viable if the mshelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Empty_RowREINHARD <string UIName=" ";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};
float Empty_REINHARD <string UIName="-----------------TONEMAP - REINHARD";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};
  bool ENABLE_HDRT <
    string UIName = "Enable HDR Toning";
  > = {true};
  bool ADAPT_HDRT <
    string UIName = "Enable Adaptation";
  > = {true};
//{
      string UIName="TONEMAP: Exposure [E]-";       string UIWidget="Spinner";  float UIMin=0.0;
    > = {1.0, 1.0, 1.0, 1.0};
    float4 fMaxLuma_E <
      string UIName="TONEMAP: Max Luminance [E]-";  string UIWidget="Spinner";  float UIMin=0.0;
    > = {1.0, 1.0, 1.0, 1.0};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msHDRTonemap(float3 inColor, float3 inBlur, float3 inAdaptation)
{
/// Time and Location interpolators ++++++++++++++++++++++++++++++++++++++++++++++++++++++///
  float fKeyValue = TODE1(TODfactors(), fKeyValue_E.x, fKeyValue_E.y, fKeyValue_E.z, fKeyValue_E.w);
  float fMaxLuma  = TODE1(TODfactors(), fMaxLuma_E.x, fMaxLuma_E.y, fMaxLuma_E.z, fMaxLuma_E.w);
///+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++///

  /// Color Space Conversion STARTS
    float3 XYZ = RGBtoXYZ(inColor.rgb);  /// Color space conversion, from RGB to XYZ
    float3 Yxy = XYZtoYxy(XYZ.xyz);      /// Color space conversion, from XYZ to Yxy

  /// Simple color intensity adjustment
    float3 lumScale = Yxy.r * fKeyValue;

  /// Adaptive Local Tonemapping
    float3 lumComp = (lumScale * (inAdaptation + lumScale / (1 * 1))) / (inAdaptation + inBlur);
      lumComp = lerp(lumScale, lumComp, ADAPT_HDRT);

  /// Global Tone-mapping
    lumComp = (lumComp * (1 + lumComp / (fMaxLuma * fMaxLuma))) / (lumComp + 1);
      Yxy.r = lerp(Yxy.r, lumComp, ENABLE_HDRT);

  /// Color Space Conversion ENDS
    XYZ.xyz      = YxytoXYZ(XYZ.xyz, Yxy.xyz);  /// Color space conversion, from Yxy to XYZ
    inColor.rgb  = XYZtoRGB(XYZ.xyz);           /// Color space conversion, from XYZ to RGB

  return inColor;
}