//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                 Contains HDR Curve                  //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Erik Reinhard: Photographic Tone Reproduction       //
// Michael Stark: Photographic Tone Reproduction       //
// Peter Shirley: Photographic Tone Reproduction       //
// James Ferwerda: Photographic Tone Reproduction      //
// Jim Hejl: Filmic ALU                                //
// Richard Burgess-Dawson: Filmic ALU                  //
// Charles Poynton: Color FAQ                          //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//

// This helper file is specifically only for use in the enbeffect.fx shader file!
// The below list is only viable if the mshelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
  bool ENABLE_ALU <
    string UIName = "Enable Filmic ALU";
  > = {true};
  bool ADAPT_ALU <
    string UIName = "Enable Adaptation";
  > = {true};
//{
    float fKeyValue <
      string UIName="TONEMAP: Exposure";     string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=5.0;
    > = {1.0};
    float fUpperTone <
      string UIName="TONEMAP: Upper Tone";   string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=10.0;
    > = {1.0};
    float fGreyPoint <
      string UIName="TONEMAP: Grey Point";   string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=1.0;
    > = {1.0};
    float fMiddleTone <
      string UIName="TONEMAP: Middle Tone";  string UIWidget="Spinner";  float UIMin=0.0;  //float UIMax=25.0;
    > = {1.0};
    float fLowerTone <
      string UIName="TONEMAP: Lower Tone";   string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=10.0;
    > = {1.0};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msHDRTonemap(float3 inColor, float3 inBlur, float3 inAdaptation)
{
  if (ENABLE_HDRALU==true)
  {
  /// Color Space Conversion STARTS
    float3 XYZ = RGBtoXYZ(inColor.rgb);  /// Color space conversion, from RGB to XYZ
    float3 Yxy = XYZtoYxy(XYZ.xyz);      /// Color space conversion, from XYZ to Yxy

/// Simple color intensity adjustment
    float3 lumScale = Yxy.r * fKeyValue;

/// Tonemapping, Local Reinhard
    float3 lumComp = (lumScale * (inAdaptation + lumScale / 1)) / (inAdaptation + inBlur);  /// Local Adaptation, affects the highlights on the screen
      lumComp = lerp(lumScale, lumComp, ADAPT_ALU);

/// Tonemapping, Global Filmic ALU
    lumComp = (lumComp * (fUpperTone * lumComp + fGreyPoint)) / (lumComp * (fUpperTone + fMiddleTone) + fLowerTone);  /// Tonemapping with Highlight and Shadow controls as well as "max allowed brightness"
      Yxy.r  = lerp(Yxy.r, lumComp, ENABLE_ALU);

  /// Color Space Conversion ENDS
    XYZ.xyz      = YxytoXYZ(XYZ.xyz, Yxy.xyz);  /// Color space conversion, from Yxy to XYZ
    inColor.rgb  = XYZtoRGB(XYZ.xyz);           /// Color space conversion, from XYZ to RGB
  }

  return inColor;
}