//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                 Contains HDR Combo                  //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Erik Reinhard: Photographic Tone Reproduction       //
// Michael Stark: Photographic Tone Reproduction       //
// Peter Shirley: Photographic Tone Reproduction       //
// James Ferwerda: Photographic Tone Reproduction      //
// John Hable: Filmic Uncharted2D                      //
// Charles Poynton: Color FAQ                          //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//

// This helper file is specifically only for use in the enbeffect.fx shader file!
// The below list is only viable if the mshelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
  bool ENABLE_U2D <
    string UIName = "Enable Filmic Uncharted2D";
  > = {true};
  bool ADAPT_U2D <
    string UIName = "Enable Adaptation";
  > = {true};
//{
    float4 fKeyValue_E <
        string UIName="TONEMAP: Exposure [E]-";     string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=5.0;  float UIStep=0.001;
    > = {3.0, 4.0, 3.0, 1.0};
    float ShoulderStrength <
        string UIName="TONEMAP: Shoulder Strength";  string UIWidget="Spinner";  float UIMin=0.001;  float UIMax=2.0;
    > = {0.251};
    float LinearStrength <
        string UIName="TONEMAP: Linear Strength";    string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=5.0;
    > = {1.0};
    float LinearAngle <
        string UIName="TONEMAP: Linear Angle";       string UIWidget="Spinner";  float UIMin=0.001;  float UIMax=1.0;
    > = {0.1};
    float ToeStrength <
        string UIName="TONEMAP: Toe Strength";       string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=2.0;
    > = {0.2};
    float ToeNumerator <
        string UIName="TONEMAP: Toe Numerator";      string UIWidget="Spinner";  float UIMin=0.001;  float UIMax=0.5;  float UIStep=0.001;
    > = {0.001};
    float ToeDenominator <
        string UIName="TONEMAP: Toe Denominator";    string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=2.0;
    > = {0.1};
    float LinearWhite <
        string UIName="TONEMAP: Linear White";       string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=50.0;
    > = {50.0};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msHDRTonemap(float3 inColor, float3 inBlur, float3 inAdaptation)
{
/// Time and Location interpolators ++++++++++++++++++++++++++++++++++++++++++++++++++++++///
  float KEY = TODE1(TODfactors(), fKeyValue_E.x, fKeyValue_E.y, fKeyValue_E.z, fKeyValue_E.w);
  float A = ShoulderStrength;
  float B = LinearStrength;
  float C = LinearAngle;
  float D = ToeStrength;
  float E = ToeNumerator;
  float F = ToeDenominator;
  float W = LinearWhite;
///+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++///

  /// Color Space Conversion STARTS
    float3 XYZ = RGBtoXYZ(inColor.rgb);  /// Color space conversion, from RGB to XYZ
    float3 Yxy = XYZtoYxy(XYZ.xyz);      /// Color space conversion, from XYZ to Yxy

/// Simple color intensity adjustment
    float3 lumScale = Yxy.r * KEY;

/// Adaptive Local Tonemapping
    float3 lumComp = (lumScale * (inAdaptation + lumScale / 1)) / (inAdaptation + inBlur);  /// Scale all luminance within a displayable range of 0 to 1
      lumComp = lerp(lumScale, lumComp, ADAPT_U2D);

/// Tonemapping
    lumComp  = ((lumComp * (A * lumComp + C * B) + D * E) / (lumComp * (A * lumComp + B) + D * F)) - E / F;  /// Perform Filmic S-Curve tone mapping
    lumComp /= ((W * (A * W + C * B) + D * E) / (W * (A * W + B) + D * F)) - E / F;
      Yxy.r  = lerp(Yxy.r, lumComp, ENABLE_U2D);

  /// Color Space Conversion ENDS
    XYZ.xyz      = YxytoXYZ(XYZ.xyz, Yxy.xyz);  /// Color space conversion, from Yxy to XYZ
    inColor.rgb  = XYZtoRGB(XYZ.xyz);           /// Color space conversion, from XYZ to RGB

  return inColor;
}