//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//            Contains Game Post-Processing            //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// Boris: For ENBSeries and his knowledge and codes    //
// JawZ: Author and developer of this file             //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Row_NightEye <
  string UIName="--------------------------NIGHTEYE";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;
> = {0.0};

  bool USE_GAMETINT <
    string UIName = "Use Game tint as base";
  > = {true};
    float4 fNERGBTint <
      string UIName="NIGHTEYE: RGB Tint";   string UIWidget="Color";
    > = {1.0, 1.0, 1.0, 1.0};
    float4 fNEIntensity_E <
      string UIName="NIGHTEYE: Intensity [E]-";  string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=100.0;
    > = {1.0, 1.0, 1.0, 1.0};

// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float4 msNightEye(float4 inColor, float2 inCoords)
{
/// Time and Location interpolators ++++++++++++++++++++++++++++++++++++++++++++++++++++++///
  float fNEIntensity = TODE1(TODfactors(), fNEIntensity_E.x, fNEIntensity_E.y, fNEIntensity_E.z, fNEIntensity_E.w);
///+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++///

  if (Params01[2].w == 2.3 || Params01[2].w == 2.4 || Params01[2].w == 3.7)
  {
    float4 gColor = inColor;
    float4 eColor = inColor;

      float4 tint = Params01[3] * fNERGBTint;  /// Game RGB Tint color
      gColor.a    = AvgLuma(gColor.rgb).w;     /// Get luminance
      gColor.rgb  = lerp(gColor.rgb, gColor.a * tint.rgb, tint.a);

      eColor.rgb *= fNERGBTint.rgb;   /// RGB Tint color

        inColor.rgb = lerp(eColor, gColor, USE_GAMETINT);

      inColor.rgb *= fNEIntensity;
      inColor.rgb  = (inColor.rgb * (1.0f + inColor.rgb / fNEIntensity)) / (inColor.rgb + 1);
  }

  return inColor;
}


/* Still Experimental
float4 msPredatorVision(float4 inColor, float2 inCoords)
{
  if (Params01[2].w == 2.3 || Params01[2].w == 2.4 || Params01[2].w == 3.7 || tempInfo1.w==6)
  {

  float2 uv = inCoords.xy;

  float3 tc = float3(1.0, 0.0, 0.0);

    float3 pixcol = TextureColor.Sample(Sampler0, uv).rgb;
    float3 colors[3];
    colors[0] = float3(0.0, 0.0, 1.0);
    colors[1] = float3(1.0, 1.0, 0.0);
    colors[2] = float3(1.0, 0.0, 0.0);
    float lum = (pixcol.r + pixcol.g + pixcol.b) / 3.0;
    int ix = (lum < 0.5)? 0:1;
    tc = lerp(colors[ix], colors[ix+1], (lum-float(ix) * 0.5) / 0.5);

    inColor = float4(tc, 1.0);

  }
  return inColor;
}
*/