//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                Contains Bloom Control               //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// soulwynd: Author of Bloom Control                   //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx or enbbloom.fx shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Empty_Row2 <string UIName="2 ";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};
//{
    float4 fLSThres_E <
      string UIName="LIGHT SPREAD: Threshold";    string UIWidget="Spinner";  float UIMin=0.0;
    > = {1.0, 1.0, 1.0, 1.0};
    float4 fLSPwr_E <
      string UIName="LIGHT SPREAD: Power";        string UIWidget="Spinner";  float UIMin=0.0;
    > = {1.0, 1.0, 1.0, 1.0};

float Empty_Row3 <string UIName="3 ";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};
    float4 fGThres_E_E <
      string UIName="GLOW: Threshold";            string UIWidget="Spinner";  float UIMin=0.0;
    > = {1.0, 1.0, 1.0, 1.0};
    float4 fGPwr_E <
      string UIName="GLOW: Power";                string UIWidget="Spinner";  float UIMin=0.0;
    > = {1.0, 1.0, 1.0, 1.0};

float Empty_Row4 <string UIName="4 ";  string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;> = {0.0};
    float4 fMaxWhite_E <
        string UIName="COLOR: Max White";         string UIWidget="Spinner";  float UIMin=0.01;  float UIMax=25.0;
    > = {1.0, 1.0, 1.0, 1.0};
    float3 fColorFilter_ESs <
        string UIName="COLOR: RGB Filter [ESs]";  string UIWidget ="Color";
    > = {1.0, 1.0, 1.0};
    float3 fColorFilter_ED <
        string UIName="COLOR: RGB Filter [ED]";   string UIWidget ="Color";
    > = {1.0, 1.0, 1.0};
    float3 fColorFilter_ESr <
        string UIName="COLOR: RGB Filter [ESr]";  string UIWidget ="Color";
    > = {1.0, 1.0, 1.0};
    float3 fColorFilter_EN <
        string UIName="COLOR: RGB Filter [EN]";   string UIWidget ="Color";
    > = {1.0, 1.0, 1.0};
    float4 fContrast_E <
        string UIName="COLOR: Contrast";          string UIWidget="Spinner";  float UIMin=1.0;  float UIMax=10.0;
    > = {1.0, 1.0, 1.0, 1.0};
    float4 fSaturation_E <
        string UIName="COLOR: Saturation";        string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=5.0;
    > = {1.0, 1.0, 1.0, 1.0};
//}





// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msBloomControl(float3 inBloom, float2 inCoord)
{
/// Time and Location interpolators ++++++++++++++++++++++++++++++++++++++++++++++++++++++///
  float  fLightSpreadThreshold = TODE1(TODfactors(), fLSThres_E.x, fLSThres_E.y, fLSThres_E.z, fLSThres_E.w);
  float  fLightSpreadPower     = TODE1(TODfactors(), fLSPwr_E.x, fLSPwr_E.y, fLSPwr_E.z, fLSPwr_E.w);
  float  fGlowThreshold        = TODE1(TODfactors(), fGThres_E_E.x, fGThres_E_E.y, fGThres_E_E.z, fGThres_E_E.w);
  float  fGlowPower            = TODE1(TODfactors(), fGPwr_E.x, fGPwr_E.y, fGPwr_E.z, fGPwr_E.w);
  float  fMaxWhite             = TODE1(TODfactors(), fMaxWhite_E.x, fMaxWhite_E.y, fMaxWhite_E.z, fMaxWhite_E.w);
  float3 fColorFilter          = TODE3(TODfactors(), fColorFilter_ESs, fColorFilter_ED, fColorFilter_ESr, fColorFilter_EN);
  float  fContrast             = TODE1(TODfactors(), fContrast_E.x, fContrast_E.y, fContrast_E.z, fContrast_E.w);
  float  fSaturation           = TODE1(TODfactors(), fSaturation_E.x, fSaturation_E.y, fSaturation_E.z, fSaturation_E.w);
///+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++///


  float3 orgBloom = inBloom;

/// Bloom Lightspread
    inBloom.xyz += max(0.0, (inBloom.xyz - fLightSpreadThreshold * 0.1) * fLightSpreadPower * (1.0 - orgBloom.xyz));

/// Bloom Glow
    inBloom.xyz += max(0.0, (inBloom.xyz - fGlowThreshold * 0.1) * fGlowPower);

/// Compress and scale bloom
    inBloom.xyz = (inBloom.xyz * (1.0f + inBloom.xyz / (fMaxWhite * fMaxWhite))) / (1.0f + inBloom.xyz); /// Scale bloom luminance within a displayable range of 0 to 1

/// Color Filter
    inBloom.xyz *= fColorFilter;

/// Contrast
    inBloom.xyz = pow(inBloom.xyz, fContrast);

/// Luminance Saturation
    float3 avgBloomLuma = AvgLuma(inBloom.xyz).w;  /// Average the scene bloom luminance
      inBloom.xyz       = lerp(avgBloomLuma.xyz, inBloom.xyz, fSaturation);

  return inBloom;
}