//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//                   Contains Sepia                    //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// Nvidia: Author of Sepia Tone                        //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the enbeffect.fx shader file!
// The below list is only viable if the mshelpers.fxh is loaded/included into the enbeffect.fx file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //
  bool ENABLE_SEPIA <
    string UIName = "Enable SEPIA";
  > = {true};
//{
    float3 fLightColor <
      string UIName="SEPIA-TONE: Light Color";    string UIWidget="Color";
    > = {1.0, 0.9, 0.5};
    float3 fDarkColor <
      string UIName="SEPIA-TONE: Dark Color";     string UIWidget="Color";
    > = {0.2, 0.05, 0.0};
    float fDesat <
      string UIName="SEPIA-TONE: Desaturate";     string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=10.0;
    > = {0.4};
    float fToned <
      string UIName="SEPIA-TONE: Tone Strength";  string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=10.0;
    > = {0.1};
//}


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //




// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

float3 msSepia(float3 inColor)
{
    float3 scnColor = fLightColor * inColor.xyz;
    float3 grayXfer = AvgLuma(inColor.xyz).w;
    float gray      = dot(grayXfer, scnColor);
    float3 muted    = lerp(scnColor, gray, fDesat);
    float3 sepia    = lerp(fDarkColor, fLightColor, gray);
    float3 result   = lerp(muted, sepia, fToned);
    inColor.xyz     = lerp(inColor.xyz, result.xyz, ENABLE_SEPIA);

  return inColor;
}