//+++++++++++++++++++++++++++++++++++++++++++++++++++++//
//               Contains Letterbox Bars               //
//           used by the Modular Shader files          //
//-----------------------CREDITS-----------------------//
// JawZ: Author and developer of this file             //
// MTichenor: Author of Letterbox                      //
//+++++++++++++++++++++++++++++++++++++++++++++++++++++//


// This helper file is specifically only for use in the effect.txt shader file!
// The below list is only viable if the msHelpers.fxh is loaded/included into the effect.txt file!


/***List of available fetches**********************************************
 * - PI // value of PI                                                    *
 * - TODEI1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODEI3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN, Var_ISr, Var_ID, Var_ISs, Var_IN)
 * - TODE1(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - TODE3(TODfactors(), Var_ESr, Var_ED, Var_ESs, Var_EN)                *
 * - DNEIFactor(Var_ED, Var_EN, Var_ID, Var_IN)                           *
 * - DNIFactor(Var_D, Var_N, Var_I)                                       *
 * - GreyScale(_s4, IN.txcoord0.xy)                                       *
 * - AvgLuma(color.rgb).x  // or .y or .z or .w, never ever .xyzw!        *
 * - LogLuma(color.rgb)                                                   *
 * - random(uv.xy)                                                        *
 * - RGBtoXYZ(color.rgb)                                                  *
 * - XYZtoYxy(XYZ.xyz)                                                    *
 * - YxytoXYZ(XYZ.xyz, Yxy.rgb)                                           *
 * - XYZtoRGB(XYZ.xyz)                                                    *
 * - RGBToHSL(color.rgb)                                                  *
 * - HSLToRGB(hsl.rgb)                                                    *
 * - RGBtoHSV(color.rgb)                                                  *
 * - HSVtoRGB(hsv.rgb)                                                    *
 * - BlendLuma(hslbase.rgb, hslblend.rgb)                                 *
 * - linearDepth(Depth, fFromFarDepth, fFromNearDepth)                    *
 * - FuncBlur(inputtex, IN.txcoord0.xy, srcsize, destsize)                *
 * - SplitScreen(_s0, color, IN.txcoord0.xy, fSplitscreenPos)             *
 * - ClipMode(color.rgb)                                                  *
 * - ShowDepth(color.rgb, IN.txcoord0.xy, fFromFarDepth, fFromNearDepth   *
 **************************************************************************/


// ------------------- //
//   GUI ANNOTATIONS   //
// ------------------- //

float Empty_Row7 <
  string UIName="F--------------------------F";   string UIWidget="spinner";  float UIMin=0.0;  float UIMax=0.0;
> = {0.0};


    bool ENABLE_LETTERBOX <
        string UIName = "Enable Letterbox Bars";
    > = {false};
    float fLetterboxBarHeight <
        string UIName="Letterbox: Height in %";   string UIWidget="Spinner";  float UIMin=0.0;  float UIMax=0.5;  float UIStep=0.001;
    > = {0.12};


// ------------------- //
//   HELPER CONSTANTS  //
// ------------------- //



// ------------------- //
//   HELPER FUNCTIONS  //
// ------------------- //

/// Simple Letterbox Bars
float4 msLetterbox(float4 inColor, float2 inTexCoords)
{
  if (ENABLE_LETTERBOX==true)
  {
    if (inTexCoords.y > 1.0f - fLetterboxBarHeight || inTexCoords.y  < fLetterboxBarHeight) inColor = float4(0.0f, 0.0f, 0.0f, 0.0f);
  }

  return inColor;
}
